from eval_kohkubo import Evaluation


def printFile(str):
    with open("./test_w.txt", mode='a') as f:
        f.write(f"{str}")

class Player():
    def __init__(self, p):
        self.p = p
        self.char = 'o' if self.p == "p1" else 'x'
        self.enemy_char = 'x' if self.char == 'o' else 'o'
        self.board = None
        self.token = None
        self.eval = Evaluation()

    def check_overlap(self, x, y):
        token = self.token
        overlap_counter = 0
        for token_y in range(token.y):
            for token_x in range(token.x):
                if self.board.board[y + token_y][x + token_x] in (
                    self.enemy_char,
                    self.enemy_char.upper(),
                ):
                    return True
                if token.shape[token_y][token_x] == '*' and self.board.board[
                    y + token_y
                ][x + token_x] in (self.char, self.char.upper()):
                        overlap_counter += 1
        if overlap_counter != 1:
            return True
        return False

    def check_overflow(self, x, y):
        token = self.token
        board = self.board
        if ((x + token.x) > board.x) or ((y + token.y) > board.y):
            return True
        if (x < 0) or (y < 0):
            return True
        return False

    def put_token(self, token_y, token_x):
        ansDict = {}
        board = self.board
        for board_y in range(board.y):
            for board_x in range(board.x):
                if board.board[board_y][board_x] in (self.char, self.char.upper()):
                    x = board_x - token_x
                    y = board_y - token_y
                    if self.check_overflow(x, y) == 0 and self.check_overlap(x, y) == 0:
                        ansDict[(y, x,)] = 0
        if len(ansDict) != 0:
            self.eval.choice_place(ansDict, self.board, self.enemy_char)
            return True
        else:
            return False

    def put_random(self):
        for token_y, token_x in self.token.get_topleft_edge():
            if token_y is None or token_x is None:
                break
            if self.put_token(token_y, token_x):
                return True
        print("0 0")
        return False



import numpy as np


def printFile(str):
    with open("./test_w.txt", mode='a') as f:
        f.write(f"{str}")

class Evaluation():
    def read_enemy_pos(self, now_board, enemy_char):
        disDict = {}
        board = now_board.board
        enemyDict = {}
        for y in range(len(board)):
            for x in range(len(board[0])):
                if enemy_char.lower() == board[y][x].lower():
                    enemyDict[(y, x,)] = 0

        for y in range(len(board)):
            for x in range(len(board[0])):
                a = np.array([y, x])
                disDict[(y, x,)] = 0
                dis_tmp = 10000
                for i in enemyDict.items():
                    b = np.array([i[0][0], i[0][1]])
                    dis = np.linalg.norm(a - b, ord=1)
                    dis_tmp = min(dis, dis_tmp)
                disDict[(y, x,)] = dis_tmp
        return disDict

    def choice_place(self, ansDict, board, enemy_char):
        evalBoardDict = self.read_enemy_pos(board, enemy_char)
        for i in ansDict.keys():
            ansDict[i] = evalBoardDict[i]
        tmp = min(ansDict, key=ansDict.get)
        printFile(f"{tmp}\n")
        print(f"{tmp[0]} {tmp[1]}")



#!/usr/bin/env python
import random

class Player():
    def __init__(self, p: str):
        self.p = p
        self.char = 'o' if self.p == "p1" else 'x'
        self.enemy_char = 'x' if self.char == 'o' else 'o'
        self.board = None
        self.token = None
        # self.e = Evaluation()

    def put_token(self, token_y: int, token_x: int):
        ansList = []
        ansDict = {}
        board = self.board
        for board_y in range(board.y):
            for board_x in range(board.x):
                if board.board[board_y][board_x] == self.char:
                    x = board_x - token_x
                    y = board_y - token_y
                    if board.check_overflow(x, y, self.token) == 0 and board.check_overlap(x, y, self.token) == 0:
                        # ansList.append(f"{y} {x}")
                        ansList.append((y, x,))
                        ansDict[(y, x,)] = 0
        if len(ansList) != 0:
            index = random.randrange(0, len(ansList))
            # self.e.eval(ansDict)
            print(f"{ansList[index][0]} {ansList[index][1]}")
            return True
        else:
            return False

    def put_random(self):
        for token_y, token_x in self.token.get_topleft_edge():
            if self.put_token(token_y, token_x):
                return True
        print("0 0")
        return False

class Evaluation():
    def eval(self, ansDict):
        for i in ansDict.keys():
            path_w = './test_w.txt'
            with open(path_w, mode='a') as f:
                f.write("'========'\n")
                f.write(f"{i}\n")
                f.write(f"{i[0]}\n")

